#pragma once

#include <map>
#include <string>

#include <Eigen/Core>

#include <RobotAPI/libraries/armem_robot_state/aron/Proprioception.aron.generated.h>

#include "RobotUnitConverterInterface.h"


namespace armarx::armem::server::robot_state
{

    class Setter
    {

    };

    class Vector3Setter
    {

    };


    class Armar6RobotUnitConverter : public RobotUnitConverterInterface
    {
        struct Value;

    public:

        Armar6RobotUnitConverter();
        virtual ~Armar6RobotUnitConverter() override;


        arondto::Proprioception convert(
            const RobotUnitDataStreaming::TimeStep& data,
            const RobotUnitDataStreaming::DataStreamingDescription& description) override;


    protected:

        void process(arondto::Proprioception dto, const std::string& entryName, const Value& value);



    private:

        struct Value
        {
            const RobotUnitDataStreaming::TimeStep& data;
            const RobotUnitDataStreaming::DataEntry& entry;
        };

        void processPlatformEntry(
            prop::arondto::Platform& dto,
            const std::string& fieldName,
            const Value& value);

        void processForceTorqueEntry(
            std::map<std::string, prop::arondto::ForceTorque>& fts,
            const std::vector<std::string>& split,
            const Value& value);

        void processForceTorqueEntry(
            prop::arondto::ForceTorque& ft,
            const std::vector<std::string>& split,
            const Value& value);

        bool processJointEntry(
            prop::arondto::Joints& dto,
            const std::vector<std::string>& split,
            const Value& value);


    private:

        template <class T>
        static T getValueAs(const Value& value);


    private:

        std::map<std::string, std::function<void(Eigen::Vector3f&, float)> > vector3fSetters;

        std::map<std::string, std::function<std::map<std::string, float>*(prop::arondto::Joints&)> > jointGetters;
        std::map<std::string, std::function<std::map<std::string, Eigen::Vector3f>*(prop::arondto::Joints&)> > jointVectorGetters;

        using JointSetter = std::function<void(prop::arondto::Joints& dto, const std::vector<std::string>& split, const Value& value)>;
        std::map<std::string, JointSetter> jointSetters;

        std::map<std::string, std::function<Eigen::Vector3f*(prop::arondto::Platform&)> > platformPoseGetters;
        std::set<std::string> platformIgnored;

        std::map<std::string, std::function<Eigen::Vector3f*(prop::arondto::ForceTorque&)> > ftGetters;


        std::map<std::string, std::string> sidePrefixMap
        {
            { "R", "Right" },
            { "L", "Left" },
        };

    };
}

