/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Pair.h"


namespace armarx::aron::cppserializer::serializer
{
    PairSerializer::PairSerializer(const typenavigator::PairNavigatorPtr& e) :
        detail::ContainerSerializerBase<typenavigator::PairNavigator, PairSerializer>("std::pair<" + ExtractCppTypename(e->getFirstAcceptedType()) + ", " + ExtractCppTypename(e->getSecondAcceptedType()) + ">",
                simox::meta::get_type_name(typeid(data::AronList)),
                simox::meta::get_type_name(typeid(type::AronPair)), e)
    {
    }

    CppBlockPtr PairSerializer::getResetHardBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine(accessor + " = " + this->getFullTypenameGenerator() + "();");
        return ResolveMaybeResetHardBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr PairSerializer::getResetSoftBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        std::string resolved_accessor = ResolveMaybeAccessor(accessor, typenavigator);

        auto child_s1 = FromAronTypeNaviagtorPtr(typenavigator->getFirstAcceptedType());
        CppBlockPtr b21 = child_s1->getResetSoftBlock(accessor + nextEl() + "first");
        block_if_data->appendBlock(b21);

        auto child_s2 = FromAronTypeNaviagtorPtr(typenavigator->getSecondAcceptedType());
        CppBlockPtr b22 = child_s2->getResetSoftBlock(accessor + nextEl() + "second");
        block_if_data->appendBlock(b22);
        return ResolveMaybeResetSoftBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr PairSerializer::getWriteTypeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("w.writeStartPair({" + MAYBE_TO_STRING(typenavigator->getMaybe()) + "}); // of " + accessor);

        std::string escaped_accessor = EscapeAccessor(accessor);

        auto child_s1 = FromAronTypeNaviagtorPtr(typenavigator->getFirstAcceptedType());
        std::string accessor_iterator1 = escaped_accessor + PAIR_FIRST_ACCESSOR;
        b->addLine("w.writeKey(\"0\");");
        CppBlockPtr b21 = child_s1->getWriteTypeBlock(accessor_iterator1);
        b->appendBlock(b21);

        auto child_s2 = FromAronTypeNaviagtorPtr(typenavigator->getSecondAcceptedType());
        std::string accessor_iterator2 = escaped_accessor + PAIR_SECOND_ACCESSOR;
        b->addLine("w.writeKey(\"1\");");
        CppBlockPtr b22 = child_s2->getWriteTypeBlock(accessor_iterator2);
        b->appendBlock(b22);

        b->addLine("w.writeEndPair(); // of " + accessor);
        return b;
    }

    CppBlockPtr PairSerializer::getWriteBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine("w.writeStartList(); // of " + accessor);

        auto child_s1 = FromAronTypeNaviagtorPtr(typenavigator->getFirstAcceptedType());
        CppBlockPtr b21 = child_s1->getWriteBlock(accessor + nextEl() + "first");
        block_if_data->appendBlock(b21);

        auto child_s2 = FromAronTypeNaviagtorPtr(typenavigator->getSecondAcceptedType());
        CppBlockPtr b22 = child_s2->getWriteBlock(accessor + nextEl() + "second");
        block_if_data->appendBlock(b22);

        block_if_data->addLine("w.writeEndList(); // of " + accessor);

        return ResolveMaybeWriteBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr PairSerializer::getReadBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());

        auto child_s1 = FromAronTypeNaviagtorPtr(typenavigator->getFirstAcceptedType());
        CppBlockPtr b21 = child_s1->getReadBlock(accessor + nextEl() + "first");
        block_if_data->appendBlock(b21);

        auto child_s2 = FromAronTypeNaviagtorPtr(typenavigator->getSecondAcceptedType());
        CppBlockPtr b22 = child_s2->getReadBlock(accessor + nextEl() + "second");
        block_if_data->appendBlock(b22);

        block_if_data->addLine("r.readEndList(); // of " + accessor);
        return ResolveMaybeReadBlock(accessor, "r.readStartList()", block_if_data, typenavigator);
    }

    CppBlockPtr PairSerializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))");
        block_if_data->addLineAsBlock("return false;");
        return ResolveMaybeEqualsBlock(accessor, otherInstanceAccessor, block_if_data, typenavigator);
    }
}

