/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <map>

// ArmarX
#include <RobotAPI/libraries/aron/core/Concepts.h>
#include <RobotAPI/libraries/aron/core/navigator/type/AllNavigators.h>

namespace armarx
{
    class RapidXmlReaderNode;
}

namespace armarx::aron::xmltypereader
{
    class Data
    {
    private:
        Data() = delete;

    public:
        // Definition tags
        static constexpr const char* TYPE_DEFINITION_TAG = "arontypedefinition";

        static constexpr const char* CODE_INCLUDES_TAG = "codeincludes";
        static constexpr const char* INCLUDES_TAG = "aronincludes";
        static constexpr const char* GENERATE_TYPES_TAG = "generatetypes";
        static constexpr const char* AUTO_CODE_INCLUDE = "autoinclude";

        static constexpr const char* INCLUDE_TAG = "include";
        static constexpr const char* GENERATE_OBJECT_TAG = "object";
        static constexpr const char* GENERATE_INT_ENUM_TAG = "intenum";
        static constexpr const char* GENERATE_NDARRAY_TAG = "ndarray";

        // Attribute names
        static constexpr const char* METHOD_ATTRIBUTE_NAME = "method";
        static constexpr const char* RETURN_ATTRIBUTE_NAME = "return";
        static constexpr const char* ARGUMENT_TYPE_ATTRIBUTE_NAME = "argumenttype";
        static constexpr const char* INCLUDE_ATTRIBUTE_NAME = "include";
        static constexpr const char* READER_ATTRIBUTE_NAME = "reader";
        static constexpr const char* WRITER_ATTRIBUTE_NAME = "writer";
        static constexpr const char* EXTENDS_ATTRIBUTE_NAME = "extends";
        static constexpr const char* NAME_ATTRIBUTE_NAME = "name";
        static constexpr const char* VALUE_ATTRIBUTE_NAME = "value";
        static constexpr const char* KEY_ATTRIBUTE_NAME = "key";
        static constexpr const char* TYPE_ATTRIBUTE_NAME = "type";
        static constexpr const char* WIDTH_ATTRIBUTE_NAME = "width";
        static constexpr const char* HEIGHT_ATTRIBUTE_NAME = "height";
        static constexpr const char* ROWS_ATTRIBUTE_NAME = "rows";
        static constexpr const char* COLS_ATTRIBUTE_NAME = "cols";
        static constexpr const char* DIMENSIONS_ATTRIBUTE_NAME = "dimensions";
        static constexpr const char* OPTIONAL_NAME = "optional";
        static constexpr const char* RAW_PTR_NAME = "raw_ptr";
        static constexpr const char* SHARED_PTR_NAME = "shared_ptr";
        static constexpr const char* UNIQUE_PTR_NAME = "unique_ptr";
        static constexpr const char* PACKAGE_NAME = "package";
        static constexpr const char* DOC_BRIEF_NAME = "doc-brief";
        static constexpr const char* DOC_AUTHOR_NAME = "doc-author";
        static constexpr const char* DOC_PARAM_NAME = "doc-param";

        // Second level tags. Only important if in specific top level tag
        static constexpr const char* OBJECT_CHILD_TAG = "objectchild";
        static constexpr const char* ENUM_VALUE_TAG = "enumvalue";

        // Top Level type tags
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    static constexpr const char* GENERATE_##capsType##_MEMBER_TAG = #lowerType;

        HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO

        static void EnforceAttribute(const RapidXmlReaderNode& node, const std::string& att);

        static bool HasAttribute(const RapidXmlReaderNode& node, const std::string& att);

        static std::string GetAttribute(const RapidXmlReaderNode& node, const std::string& att);

        static std::string GetAttributeWithDefault(const RapidXmlReaderNode& node, const std::string& att, const std::string def);

        static bool AttributeIsTrue(const RapidXmlReaderNode& node, const std::string& att);

        static bool HasTagName(const RapidXmlReaderNode& node, const std::string& name);

        static void EnforceTagName(const RapidXmlReaderNode& node, const std::string& name);

        static std::string GetTagName(const RapidXmlReaderNode& node);

        static bool CheckMinChildSize(const RapidXmlReaderNode& node, const size_t size);

        static bool CheckMaxChildSize(const RapidXmlReaderNode& node, const size_t size);

        static bool CheckExactChildSize(const RapidXmlReaderNode& node, const size_t size);

        static void EnforceChildSize(const RapidXmlReaderNode& node, const size_t size);
    };
}
