/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/core/io/typeIO/Reader.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/io/typeIO/reader/nlohmannJSON/NlohmannJSONReaderToken.h>

namespace armarx::aron::typeIO::reader
{
    class NlohmannJSONReader;
    typedef std::shared_ptr<NlohmannJSONReader> NlohmannJSONReaderPtr;

    class NlohmannJSONReader :
        virtual public ReaderInterface
    {
    public:
        using PointerType = NlohmannJSONReaderPtr;

    public:
        // constructors
        NlohmannJSONReader() = delete;
        NlohmannJSONReader(const nlohmann::json& n);
        NlohmannJSONReader(const std::string& n);

        virtual ReadStartObjectReturnType readStartObject() override;
        virtual bool readEndObject() override;
        virtual ReadStartListReturnType readStartList() override;
        virtual bool readEndList() override;
        virtual ReadStartDictReturnType readStartDict() override;
        virtual bool readEndDict() override;
        virtual ReadStartTupleReturnType readStartTuple() override;
        virtual bool readEndTuple() override;
        virtual ReadStartPairReturnType readStartPair() override;
        virtual bool readEndPair() override;

        virtual ReadEigenMatrixReturnType readEigenMatrix() override;
        virtual ReadEigenQuaternionReturnType readEigenQuaternion() override;
        virtual ReadIVTCbyteImageReturnType readIVTCByteImage() override;
        virtual ReadOpenCVMatReturnType readOpenCVMat() override;
        virtual ReadPCLPointCloudReturnType readPCLPointCloud() override;
        virtual ReadPositionReturnType readPosition() override;
        virtual ReadOrientationReturnType readOrientation() override;
        virtual ReadPoseReturnType readPose() override;

        virtual ReadPrimitiveReturnType readInt() override;
        virtual ReadPrimitiveReturnType readLong() override;
        virtual ReadPrimitiveReturnType readFloat() override;
        virtual ReadPrimitiveReturnType readDouble() override;
        virtual ReadPrimitiveReturnType readString() override;
        virtual ReadPrimitiveReturnType readBool() override;
        virtual ReadPrimitiveReturnType readTime() override;

        virtual std::string readKey() override;
        virtual void loadMember(const std::string&) override;

        // Helper functions
        virtual type::Descriptor getTypeOfNext() const override;

    private:
        nlohmann::json getNextAndIncrease();

    private:
        nlohmann::json input;
        bool readInitialStart = false;
        std::stack<NlohmannJSONReaderTokenPtr> stack;
    };
}
